import pytest
import numpy as np

from contextlib import contextmanager
from irmetrics.topk import rr, recall, precision, ndcg, ap


@contextmanager
def does_not_raise():
    yield


@pytest.fixture
def inputs():
    return [
        (1, [1, 0, 0]),
        (1, [0, 1, 0]),
        (1, [0, 0, 1]),
        (1, [1, 1, 1]),
        (1, [0, 0, 0]),
        (1, [0, 0, 0]),
    ]


_OUTPUTS = {
    rr: [
        (1, does_not_raise),
        (1. / 2, does_not_raise),
        (1. / 3, does_not_raise),
        (1., does_not_raise),
        (0, does_not_raise),
        (0, does_not_raise),
    ],
    recall: [
        (True, does_not_raise),
        (True, does_not_raise),
        (True, does_not_raise),
        (True, does_not_raise),
        (False, does_not_raise),
        (False, does_not_raise),
    ],
    precision: [
        (1. / 3., does_not_raise),
        (1. / 3., does_not_raise),
        (1. / 3., does_not_raise),
        (1. / 3., does_not_raise),
        (False, does_not_raise),
        (False, does_not_raise),
    ],
    ndcg: [
        (1., does_not_raise),
        (1. / np.log2(2 + 1), does_not_raise),  # idx + 1, (idx starts from 1)
        (1. / np.log2(3 + 1), does_not_raise),  # idx + 1, (idx starts from 1)
        (1., does_not_raise),
        (np.nan, does_not_raise),
        (np.nan, does_not_raise),
    ],
    ap: [
        (1. / 3., does_not_raise),
        (0., does_not_raise),
        (0., does_not_raise),
        (1. / 3., does_not_raise),
        (False, does_not_raise),
        (False, does_not_raise),
    ],
}


@pytest.fixture
def raw_outputs(measure):
    return list(zip(*_OUTPUTS[measure]))


@pytest.fixture
def expected(raw_outputs):
    return raw_outputs[0]


@pytest.fixture
def exceptions(raw_outputs):
    return raw_outputs[1]



import pytest
import numpy as np

from irmetrics.topk import rr, recall, precision, ndcg, ap
from irmetrics.relevance import unilabel, multilabel


@pytest.fixture
def cases(inputs, expected, exceptions):
    return zip(inputs, expected, exceptions)


@pytest.mark.parametrize("measure", [
    rr,
    recall,
    precision,
    ndcg,
    ap,
])
@pytest.mark.parametrize("relevance", [
    unilabel,
    multilabel,
])
def test_all(cases, measure, relevance):
    for (y_true, y_pred), expected, exception in cases:
        with exception():
            np.testing.assert_equal(
                measure(y_true, y_pred, relevance=relevance),
                expected
            )


@pytest.mark.parametrize("measure", [
    rr,
    recall,
    precision,
    ndcg,
    ap,
])
@pytest.mark.parametrize("relevance", [
    unilabel,
    multilabel,
])
def test_all_vectorized(cases, measure, relevance, n_samples=128):
    for (y_true, y_pred), expected, exception in cases:

        # Repeat the inputs and expected values along the batch dimension
        y_trues = np.repeat(np.array(y_true), n_samples)
        y_preds = np.repeat(np.atleast_2d(y_pred), n_samples, axis=0)
        outputs = np.repeat(np.array(expected), n_samples)

        with exception():
            np.testing.assert_equal(
                measure(y_trues, y_preds, relevance=relevance),
                outputs
            )
